@@ -28,7 +28,7 @@ function getPropTypeWarningMessage(propTypes, object, componentName) {
   }
   resetWarningCache();
 
-  PropTypes.checkPropTypes(propTypes, object, 'prop', 'testComponent');
+  PropTypes.checkPropTypes(propTypes, object, 'prop', componentName);
   const callCount = console.error.calls.count();
   if (callCount > 1) {
     throw new Error('Too many warnings.');
@@ -112,7 +112,7 @@ describe('PropTypesDevelopmentReact15', () => {
 
   describe('checkPropTypes', () => {
     it('should warn for invalid validators', () => {
-      spyOn(console, 'error')
+      spyOn(console, 'error');
       const propTypes = { foo: undefined };
       const props = { foo: 'foo' };
       PropTypes.checkPropTypes(
@@ -128,6 +128,51 @@ describe('PropTypesDevelopmentReact15', () => {
       );
     });
 
+    it('should only warn once for identical validator failures', () => {
+      spyOn(console, 'error');
+      const propTypes = { foo: undefined };
+      const props = { foo: 'foo' };
+      PropTypes.checkPropTypes(
+        propTypes,
+        props,
+        'prop',
+        'testComponent',
+        null,
+      );
+      PropTypes.checkPropTypes(
+        propTypes,
+        props,
+        'prop',
+        'testComponent',
+        null,
+      );
+      expect(console.error.calls.count()).toEqual(1);
+    });
+
+    describe('checkPropTypes.resetWarningCache', () => {
+      it('should reset warning cache', () => {
+        spyOn(console, 'error');
+        const propTypes = { foo: undefined };
+        const props = { foo: 'foo' };
+        PropTypes.checkPropTypes(
+          propTypes,
+          props,
+          'prop',
+          'testComponent',
+          null,
+        );
+        PropTypes.checkPropTypes.resetWarningCache();
+        PropTypes.checkPropTypes(
+          propTypes,
+          props,
+          'prop',
+          'testComponent',
+          null,
+        );
+        expect(console.error.calls.count()).toEqual(2);
+      });
+    });
+
     it('does not return a value from a validator', () => {
       spyOn(console, 'error');
       const propTypes = {
@@ -181,6 +226,30 @@ describe('PropTypesDevelopmentReact15', () => {
     });
   });
 
+  describe('resetWarningCache', () => {
+    it('should reset warning cache', () => {
+      spyOn(console, 'error');
+      const propTypes = { foo: undefined };
+      const props = { foo: 'foo' };
+      PropTypes.checkPropTypes(
+        propTypes,
+        props,
+        'prop',
+        'testComponent',
+        null,
+      );
+      PropTypes.resetWarningCache();
+      PropTypes.checkPropTypes(
+        propTypes,
+        props,
+        'prop',
+        'testComponent',
+        null,
+      );
+      expect(console.error.calls.count()).toEqual(2);
+    });
+  });
+
   describe('Primitive Types', () => {
     it('should warn for invalid strings', () => {
       typeCheckFail(
@@ -249,6 +318,12 @@ describe('PropTypesDevelopmentReact15', () => {
 
     it('should warn for missing required values', () => {
       typeCheckFailRequiredValues(PropTypes.string.isRequired);
+      typeCheckFailRequiredValues(PropTypes.array.isRequired);
+      typeCheckFailRequiredValues(PropTypes.symbol.isRequired);
+      typeCheckFailRequiredValues(PropTypes.number.isRequired);
+      typeCheckFailRequiredValues(PropTypes.bool.isRequired);
+      typeCheckFailRequiredValues(PropTypes.func.isRequired);
+      typeCheckFailRequiredValues(PropTypes.shape({}).isRequired);
     });
 
     it('should warn if called manually in development', () => {
@@ -734,6 +809,13 @@ describe('PropTypesDevelopmentReact15', () => {
         'Invalid prop `testProp.c` of type `string` supplied to `testComponent`, ' +
           'expected `number`.',
       );
+
+      typeCheckFail(
+        PropTypes.objectOf(PropTypes.number.isRequired),
+        {a: 1, b: 2, c: undefined},
+        'Warning: Failed prop type: The prop `testProp.c` is marked as required in `testComponent`, ' +
+          'but its value is `undefined`.'
+      );
     });
 
     it('should warn with invalid complex types', () => {
@@ -748,6 +830,13 @@ describe('PropTypesDevelopmentReact15', () => {
           name +
           '`.',
       );
+
+      typeCheckFail(
+        PropTypes.objectOf(PropTypes.instanceOf(Thing).isRequired),
+        {a: new Thing(), b: undefined},
+        'Warning: Failed prop type: The prop `testProp.b` is marked as required in `testComponent`, ' +
+          'but its value is `undefined`.'
+      );
     });
 
     it('should warn when passed something other than an object', () => {
@@ -777,10 +866,20 @@ describe('PropTypesDevelopmentReact15', () => {
       );
     });
 
+    it('should not warn when passing an object with no prototype', () => {
+      typeCheckPass(PropTypes.objectOf(PropTypes.number), Object.create(null));
+    });
+
     it('should not warn when passing an empty object', () => {
       typeCheckPass(PropTypes.objectOf(PropTypes.number), {});
     });
 
+    it('should not warn when passing an object with a hasOwnProperty property', () => {
+      typeCheckPass(PropTypes.objectOf(PropTypes.number), {
+        hasOwnProperty: 3,
+      });
+    });
+
     it('should be implicitly optional and not warn without values', () => {
       typeCheckPass(PropTypes.objectOf(PropTypes.number), null);
       typeCheckPass(PropTypes.objectOf(PropTypes.number), undefined);
@@ -815,11 +914,25 @@ describe('PropTypesDevelopmentReact15', () => {
     it('should warn but not error for invalid argument', () => {
       spyOn(console, 'error');
 
+      PropTypes.oneOf('red');
+
+      expect(console.error).toHaveBeenCalled();
+      expect(console.error.calls.argsFor(0)[0]).toContain(
+        'Invalid argument supplied to oneOf, expected an array.',
+      );
+
+      typeCheckPass(PropTypes.oneOf('red', 'blue'), 'red');
+    });
+
+    it('should warn but not error for invalid multiple arguments', () => {
+      spyOn(console, 'error');
+
       PropTypes.oneOf('red', 'blue');
 
       expect(console.error).toHaveBeenCalled();
       expect(console.error.calls.argsFor(0)[0]).toContain(
-        'Invalid argument supplied to oneOf, expected an instance of array.',
+        'Invalid arguments supplied to oneOf, expected an array, got 2 arguments. '
+        + 'A common mistake is to write oneOf(x, y, z) instead of oneOf([x, y, z]).',
       );
 
       typeCheckPass(PropTypes.oneOf('red', 'blue'), 'red');
@@ -850,6 +963,35 @@ describe('PropTypesDevelopmentReact15', () => {
         'Invalid prop `testProp` of value `false` supplied to ' +
           '`testComponent`, expected one of [0,"false"].',
       );
+      typeCheckFail(
+        PropTypes.oneOf([Symbol('red'), Symbol('blue')]),
+        Symbol('green'),
+        'Invalid prop `testProp` of value `Symbol(green)` supplied to ' +
+          '`testComponent`, expected one of ["Symbol(red)","Symbol(blue)"].',
+      );
+      typeCheckFail(
+        PropTypes.oneOf([0, 'false']).isRequired,
+        undefined,
+        'Warning: Failed prop type: The prop `testProp` is marked as required in `testComponent`, ' +
+          'but its value is `undefined`.'
+      );
+      typeCheckFail(
+        PropTypes.oneOf([0, 'false']).isRequired,
+        null,
+        'Warning: Failed prop type: The prop `testProp` is marked as required in `testComponent`, ' +
+          'but its value is `null`.'
+      );
+    });
+
+    it('does not fail when the valid types contain null or undefined', () => {
+      typeCheckFail(
+        PropTypes.oneOf([0, 'false', null, undefined]),
+        false,
+        'Warning: Failed prop type: Invalid prop `testProp` of value `false` supplied to ' +
+          '`testComponent`, expected one of [0,"false",null,null].',
+          // TODO: uncomment and fix implementation
+          // '`testComponent`, expected one of [0,"false",null,undefined].',
+      );
     });
 
     it('should not warn for valid values', () => {
@@ -1016,6 +1158,20 @@ describe('PropTypesDevelopmentReact15', () => {
         'The prop `testProp.key` is marked as required in `testComponent`, ' +
           'but its value is `undefined`.',
       );
+
+      typeCheckFail(
+        PropTypes.shape({key: PropTypes.number.isRequired}),
+        {key: undefined},
+        'The prop `testProp.key` is marked as required in `testComponent`, ' +
+          'but its value is `undefined`.',
+      );
+
+      typeCheckFail(
+        PropTypes.shape({key: PropTypes.number.isRequired}),
+        {key: null},
+        'The prop `testProp.key` is marked as required in `testComponent`, ' +
+          'but its value is `null`.',
+      );
     });
 
     it('should warn for the first required type', () => {
