@@ -18,6 +18,7 @@ function resetWarningCache() {
   React = require('react');
   PropTypes = require('../index');
 }
+resetWarningCache();
 
 function getPropTypeWarningMessage(propTypes, object, componentName) {
   if (!console.error.calls) {
@@ -27,7 +28,7 @@ function getPropTypeWarningMessage(propTypes, object, componentName) {
   }
   resetWarningCache();
 
-  PropTypes.checkPropTypes(propTypes, object, 'prop', 'testComponent');
+  PropTypes.checkPropTypes(propTypes, object, 'prop', componentName);
   const callCount = console.error.calls.count();
   if (callCount > 1) {
     throw new Error('Too many warnings.');
@@ -124,6 +125,51 @@ describe('PropTypesDevelopmentStandalone', () => {
       );
     });
 
+    it('should only warn once for identical validator failures', () => {
+      spyOn(console, 'error');
+      const propTypes = { foo: undefined };
+      const props = { foo: 'foo' };
+      PropTypes.checkPropTypes(
+        propTypes,
+        props,
+        'prop',
+        'testComponent',
+        null,
+      );
+      PropTypes.checkPropTypes(
+        propTypes,
+        props,
+        'prop',
+        'testComponent',
+        null,
+      );
+      expect(console.error.calls.count()).toEqual(1);
+    });
+
+    describe('checkPropTypes.resetWarningCache', () => {
+      it('should reset warning cache', () => {
+        spyOn(console, 'error')
+        const propTypes = { foo: undefined };
+        const props = { foo: 'foo' };
+        PropTypes.checkPropTypes(
+          propTypes,
+          props,
+          'prop',
+          'testComponent',
+          null,
+        );
+        PropTypes.checkPropTypes.resetWarningCache();
+        PropTypes.checkPropTypes(
+          propTypes,
+          props,
+          'prop',
+          'testComponent',
+          null,
+        );
+        expect(console.error.calls.count()).toEqual(2);
+      });
+    });
+
     it('does not return a value from a validator', () => {
       spyOn(console, 'error');
       const propTypes = {
@@ -177,6 +223,30 @@ describe('PropTypesDevelopmentStandalone', () => {
     });
   });
 
+  describe('resetWarningCache', () => {
+    it('should reset warning cache', () => {
+      spyOn(console, 'error');
+      const propTypes = { foo: undefined };
+      const props = { foo: 'foo' };
+      PropTypes.checkPropTypes(
+        propTypes,
+        props,
+        'prop',
+        'testComponent',
+        null,
+      );
+      PropTypes.resetWarningCache();
+      PropTypes.checkPropTypes(
+        propTypes,
+        props,
+        'prop',
+        'testComponent',
+        null,
+      );
+      expect(console.error.calls.count()).toEqual(2);
+    });
+  });
+
   describe('Primitive Types', () => {
     it('should warn for invalid strings', () => {
       typeCheckFail(
@@ -245,6 +315,12 @@ describe('PropTypesDevelopmentStandalone', () => {
 
     it('should warn for missing required values', () => {
       typeCheckFailRequiredValues(PropTypes.string.isRequired);
+      typeCheckFailRequiredValues(PropTypes.array.isRequired);
+      typeCheckFailRequiredValues(PropTypes.symbol.isRequired);
+      typeCheckFailRequiredValues(PropTypes.number.isRequired);
+      typeCheckFailRequiredValues(PropTypes.bool.isRequired);
+      typeCheckFailRequiredValues(PropTypes.func.isRequired);
+      typeCheckFailRequiredValues(PropTypes.shape({}).isRequired);
     });
 
     it('should warn if called manually in development', () => {
@@ -486,6 +562,77 @@ describe('PropTypesDevelopmentStandalone', () => {
     });
   });
 
+  describe('ElementType Types', () => {
+    it('should support native component', () => {
+      typeCheckPass(PropTypes.elementType, 'div');
+    });
+
+    it('should support stateless component', () => {
+      var MyComponent = () => <div />;
+      typeCheckPass(PropTypes.elementType, MyComponent);
+    });
+
+    it('should support stateful component', () => {
+      class MyComponent extends React.Component {
+        render() {
+          return <div />;
+        }
+      }
+      typeCheckPass(PropTypes.elementType, MyComponent);
+    });
+
+    (React.forwardRef ? it : it.skip)('should support forwardRef component', () => {
+      const MyForwardRef = React.forwardRef((props, ref) => <div ref={ref} />);
+      typeCheckPass(PropTypes.elementType, MyForwardRef);
+    });
+
+    (React.createContext ? it : it.skip)('should support context provider component', () => {
+      const MyContext = React.createContext('test');
+      typeCheckPass(PropTypes.elementType, MyContext.Provider);
+    });
+
+    (React.createContext ? it : it.skip)('should support context consumer component', () => {
+      const MyContext = React.createContext('test');
+      typeCheckPass(PropTypes.elementType, MyContext.Consumer);
+    });
+
+    it('should warn for invalid types', () => {
+      typeCheckFail(
+        PropTypes.elementType,
+        true,
+        'Invalid prop `testProp` of type `boolean` supplied to ' +
+          '`testComponent`, expected a single ReactElement type.',
+      );
+
+      typeCheckFail(
+        PropTypes.elementType,
+        {},
+        'Invalid prop `testProp` of type `object` supplied to ' +
+          '`testComponent`, expected a single ReactElement type.',
+      );
+
+      typeCheckFail(
+        PropTypes.elementType,
+        [],
+        'Invalid prop `testProp` of type `array` supplied to ' +
+          '`testComponent`, expected a single ReactElement type.',
+      );
+
+      it('should warn for missing required values', () => {
+        typeCheckFailRequiredValues(PropTypes.elementType.isRequired);
+      });
+    });
+
+    it('should warn for React element', () => {
+      typeCheckFail(
+        PropTypes.elementType,
+        <div />,
+        'Invalid prop `testProp` of type `object` supplied to ' +
+          '`testComponent`, expected a single ReactElement type.',
+      );
+    });
+  });
+
   describe('Instance Types', () => {
     it('should warn for invalid instances', () => {
       function Person() {}
@@ -723,6 +870,12 @@ describe('PropTypesDevelopmentStandalone', () => {
       });
     });
 
+    it('should support objects with a null prototype', () => {
+      const nullObj = Object.create(null);
+      nullObj.test = 'a property';
+      typeCheckPass(PropTypes.objectOf(PropTypes.string), nullObj);
+    });
+
     it('should warn with invalid items in the object', () => {
       typeCheckFail(
         PropTypes.objectOf(PropTypes.number),
@@ -730,6 +883,13 @@ describe('PropTypesDevelopmentStandalone', () => {
         'Invalid prop `testProp.c` of type `string` supplied to `testComponent`, ' +
           'expected `number`.',
       );
+
+      typeCheckFail(
+        PropTypes.objectOf(PropTypes.number.isRequired),
+        {a: 1, b: 2, c: undefined},
+        'Warning: Failed prop type: The prop `testProp.c` is marked as required in `testComponent`, ' +
+          'but its value is `undefined`.'
+      );
     });
 
     it('should warn with invalid complex types', () => {
@@ -744,6 +904,13 @@ describe('PropTypesDevelopmentStandalone', () => {
           name +
           '`.',
       );
+
+      typeCheckFail(
+        PropTypes.objectOf(PropTypes.instanceOf(Thing).isRequired),
+        {a: new Thing(), b: undefined},
+        'Warning: Failed prop type: The prop `testProp.b` is marked as required in `testComponent`, ' +
+          'but its value is `undefined`.'
+      );
     });
 
     it('should warn when passed something other than an object', () => {
@@ -773,10 +940,20 @@ describe('PropTypesDevelopmentStandalone', () => {
       );
     });
 
+    it('should not warn when passing an object with no prototype', () => {
+      typeCheckPass(PropTypes.objectOf(PropTypes.number), Object.create(null));
+    });
+
     it('should not warn when passing an empty object', () => {
       typeCheckPass(PropTypes.objectOf(PropTypes.number), {});
     });
 
+    it('should not warn when passing an object with a hasOwnProperty property', () => {
+      typeCheckPass(PropTypes.objectOf(PropTypes.number), {
+        hasOwnProperty: 3,
+      });
+    });
+
     it('should be implicitly optional and not warn without values', () => {
       typeCheckPass(PropTypes.objectOf(PropTypes.number), null);
       typeCheckPass(PropTypes.objectOf(PropTypes.number), undefined);
@@ -811,11 +988,25 @@ describe('PropTypesDevelopmentStandalone', () => {
     it('should warn but not error for invalid argument', () => {
       spyOn(console, 'error');
 
+      PropTypes.oneOf('red');
+
+      expect(console.error).toHaveBeenCalled();
+      expect(console.error.calls.argsFor(0)[0]).toContain(
+        'Invalid argument supplied to oneOf, expected an array.',
+      );
+
+      typeCheckPass(PropTypes.oneOf('red'), 'red');
+    });
+
+    it('should warn but not error for invalid multiple arguments', () => {
+      spyOn(console, 'error');
+
       PropTypes.oneOf('red', 'blue');
 
       expect(console.error).toHaveBeenCalled();
       expect(console.error.calls.argsFor(0)[0]).toContain(
-        'Invalid argument supplied to oneOf, expected an instance of array.',
+        'Invalid arguments supplied to oneOf, expected an array, got 2 arguments. '
+        + 'A common mistake is to write oneOf(x, y, z) instead of oneOf([x, y, z]).',
       );
 
       typeCheckPass(PropTypes.oneOf('red', 'blue'), 'red');
@@ -846,6 +1037,35 @@ describe('PropTypesDevelopmentStandalone', () => {
         'Invalid prop `testProp` of value `false` supplied to ' +
           '`testComponent`, expected one of [0,"false"].',
       );
+      typeCheckFail(
+        PropTypes.oneOf([Symbol('red'), Symbol('blue')]),
+        Symbol('green'),
+        'Invalid prop `testProp` of value `Symbol(green)` supplied to ' +
+          '`testComponent`, expected one of ["Symbol(red)","Symbol(blue)"].',
+      );
+      typeCheckFail(
+        PropTypes.oneOf([0, 'false']).isRequired,
+        undefined,
+        'Warning: Failed prop type: The prop `testProp` is marked as required in `testComponent`, ' +
+          'but its value is `undefined`.'
+      );
+      typeCheckFail(
+        PropTypes.oneOf([0, 'false']).isRequired,
+        null,
+        'Warning: Failed prop type: The prop `testProp` is marked as required in `testComponent`, ' +
+          'but its value is `null`.'
+      );
+    });
+
+    it('does not fail when the valid types contain null or undefined', () => {
+      typeCheckFail(
+        PropTypes.oneOf([0, 'false', null, undefined]),
+        false,
+        'Warning: Failed prop type: Invalid prop `testProp` of value `false` supplied to ' +
+          '`testComponent`, expected one of [0,"false",null,null].',
+          // TODO: uncomment and fix implementation
+          // '`testComponent`, expected one of [0,"false",null,undefined].',
+      );
     });
 
     it('should not warn for valid values', () => {
@@ -1012,6 +1232,20 @@ describe('PropTypesDevelopmentStandalone', () => {
         'The prop `testProp.key` is marked as required in `testComponent`, ' +
           'but its value is `undefined`.',
       );
+
+      typeCheckFail(
+        PropTypes.shape({key: PropTypes.number.isRequired}),
+        {key: undefined},
+        'The prop `testProp.key` is marked as required in `testComponent`, ' +
+          'but its value is `undefined`.',
+      );
+
+      typeCheckFail(
+        PropTypes.shape({key: PropTypes.number.isRequired}),
+        {key: null},
+        'The prop `testProp.key` is marked as required in `testComponent`, ' +
+          'but its value is `null`.',
+      );
     });
 
     it('should warn for the first required type', () => {
