@@ -121,6 +121,12 @@ describe('PropTypesProductionReact15', () => {
 
     it('should warn for missing required values', () => {
       expectNoop(PropTypes.string.isRequired);
+      expectNoop(PropTypes.array.isRequired);
+      expectNoop(PropTypes.symbol.isRequired);
+      expectNoop(PropTypes.number.isRequired);
+      expectNoop(PropTypes.bool.isRequired);
+      expectNoop(PropTypes.func.isRequired);
+      expectNoop(PropTypes.shape({}).isRequired);
     });
 
     it('should warn if called manually in development', () => {
@@ -606,6 +612,13 @@ describe('PropTypesProductionReact15', () => {
         'Invalid prop `testProp.c` of type `string` supplied to `testComponent`, ' +
           'expected `number`.',
       );
+
+      expectNoop(
+        PropTypes.objectOf(PropTypes.number.isRequired),
+        {a: 1, b: 2, c: undefined},
+        'Warning: Failed prop type: The prop `testProp.c` is marked as required in `testComponent`, ' +
+          'but its value is `undefined`.'
+      );
     });
 
     it('should warn with invalid complex types', () => {
@@ -620,6 +633,13 @@ describe('PropTypesProductionReact15', () => {
           name +
           '`.',
       );
+
+      expectNoop(
+        PropTypes.objectOf(PropTypes.instanceOf(Thing).isRequired),
+        {a: new Thing(), b: undefined},
+        'Warning: Failed prop type: The prop `testProp.b` is marked as required in `testComponent`, ' +
+          'but its value is `undefined`.'
+      );
     });
 
     it('should warn when passed something other than an object', () => {
@@ -649,10 +669,20 @@ describe('PropTypesProductionReact15', () => {
       );
     });
 
+    it('should not warn when passing an object with no prototype', () => {
+      expectNoop(PropTypes.objectOf(PropTypes.number), Object.create(null));
+    });
+
     it('should not warn when passing an empty object', () => {
       expectNoop(PropTypes.objectOf(PropTypes.number), {});
     });
 
+    it('should not warn when passing an object with a hasOwnProperty property', () => {
+      expectNoop(PropTypes.objectOf(PropTypes.number), {
+        hasOwnProperty: 3,
+      });
+    });
+
     it('should be implicitly optional and not warn without values', () => {
       expectNoop(PropTypes.objectOf(PropTypes.number), null);
       expectNoop(PropTypes.objectOf(PropTypes.number), undefined);
@@ -718,6 +748,33 @@ describe('PropTypesProductionReact15', () => {
         'Invalid prop `testProp` of value `false` supplied to ' +
           '`testComponent`, expected one of [0,"false"].',
       );
+      expectNoop(
+        PropTypes.oneOf([Symbol('red'), Symbol('blue')]),
+        Symbol('green'),
+        'Invalid prop `testProp` of value `Symbol(green)` supplied to ' +
+          '`testComponent`, expected one of ["Symbol(red)","Symbol(blue)"].',
+      );
+      expectNoop(
+        PropTypes.oneOf([0, 'false']).isRequired,
+        undefined,
+        'Warning: Failed prop type: The prop `testProp` is marked as required in `testComponent`, ' +
+          'but its value is `undefined`.'
+      );
+      expectNoop(
+        PropTypes.oneOf([0, 'false']).isRequired,
+        null,
+        'Warning: Failed prop type: The prop `testProp` is marked as required in `testComponent`, ' +
+          'but its value is `null`.'
+      );
+    });
+
+    it('does not fail when the valid types contain null or undefined', () => {
+      expectNoop(
+        PropTypes.oneOf([0, 'false', null, undefined]),
+        false,
+        'Warning: Failed prop type: Invalid prop `testProp` of value `false` supplied to ' +
+          '`testComponent`, expected one of [0,"false",null,undefined].',
+      );
     });
 
     it('should not warn for valid values', () => {
@@ -871,6 +928,20 @@ describe('PropTypesProductionReact15', () => {
         'The prop `testProp.key` is marked as required in `testComponent`, ' +
           'but its value is `undefined`.',
       );
+
+      expectNoop(
+        PropTypes.shape({key: PropTypes.number.isRequired}),
+        {key: undefined},
+        'The prop `testProp.key` is marked as required in `testComponent`, ' +
+          'but its value is `undefined`.',
+      );
+
+      expectNoop(
+        PropTypes.shape({key: PropTypes.number.isRequired}),
+        {key: null},
+        'The prop `testProp.key` is marked as required in `testComponent`, ' +
+          'but its value is `null`.',
+      );
     });
 
     it('should warn for the first required type', () => {
@@ -958,4 +1029,26 @@ describe('PropTypesProductionReact15', () => {
       expectNoop(PropTypes.symbol, CoreSymbol('core-js'));
     });
   });
+
+  describe('checkPropTypes', function() {
+    describe('checkPropTypes.resetWarningCache', () => {
+      it('should provide empty function', () => {
+        spyOn(console, 'error');
+
+        var spy = jest.fn();
+        PropTypes.checkPropTypes.resetWarningCache();
+        expect(spy).not.toBeCalled();
+      });
+    });
+  });
+
+  describe('resetWarningCache', () => {
+    it('should provide empty function', () => {
+      spyOn(console, 'error');
+
+      var spy = jest.fn();
+      PropTypes.resetWarningCache();
+      expect(spy).not.toBeCalled();
+    });
+  });
 });
